@@ -14,6 +14,7 @@ const {createStore} = imports.misc.state;
 const {AppMenuButtonRightClickMenu, HoverMenuController, AppThumbnailHoverMenu} = require('./menus');
 const {
     FLASH_INTERVAL,
+    FLASH_MAX_COUNT,
     MAX_BUTTON_WIDTH,
     BUTTON_BOX_ANIMATION_TIME,
     RESERVE_KEYS,
@@ -71,27 +72,29 @@ class AppGroup {
             verticalThumbs: false,
             groupReady: false,
             thumbnailMenuEntered: false,
-            fileDrag: false
+            fileDrag: false,
+            pressed: true
         });
 
         this.groupState.connect({
             isFavoriteApp: () => this.handleFavorite(true),
             getActor: () => this.actor,
-            launchNewInstance: () => this.launchNewInstance(),
+            launchNewInstance: (...args) => this.launchNewInstance(...args),
             checkFocusStyle: () => this.checkFocusStyle()
         });
 
         this.signals = new SignalManager(null);
+        this.appKeyTimeout = 0;
 
         // TODO: This needs to be in state so it can be updated more reliably.
         this.labelVisible = this.state.settings.titleDisplay !== TitleDisplay.None && this.state.isHorizontal;
-        this._progress = 0;
+        this.progress = 0;
 
         this.actor =  new Cinnamon.GenericContainer({
             name: 'appButton',
             style_class: 'grouped-window-list-item-box',
             important: true,
-            reactive: true,
+            reactive: !this.state.panelEditMode,
             can_focus: true,
             track_hover: true
         });
@@ -148,6 +151,7 @@ class AppGroup {
         this.groupState.set({tooltip: new Tooltips.PanelItemTooltip({actor: this.actor}, '', this.state.orientation)});
 
         this._draggable = new DND._Draggable(this.actor);
+        this._draggable.inhibit = !this.state.settings.enableDragging;
 
         this.signals.connect(this.actor, 'get-preferred-width', (...args) => this.getPreferredWidth(...args));
         this.signals.connect(this.actor, 'get-preferred-height', (...args) => this.getPreferredHeight(...args));
@@ -158,7 +162,6 @@ class AppGroup {
         this.signals.connect(this.actor, 'button-press-event', (...args) => this.onAppButtonPress(...args));
         this.signals.connect(this._draggable, 'drag-begin', (...args) => this.onDragBegin(...args));
         this.signals.connect(this._draggable, 'drag-cancelled', (...args) => this.onDragCancelled(...args));
-        this.signals.connect(this._draggable, 'drag-end', (...args) => this.onDragEnd(...args));
 
         this.calcWindowNumber();
         this.on_orientation_changed(true);
@@ -257,7 +260,7 @@ class AppGroup {
     setMargin() {
         let direction = this.state.isHorizontal ? 'right' : 'bottom';
         let existingStyle = this.actor.style ? this.actor.style : '';
-        this.actor.style = existingStyle + 'margin-' + direction + ': ' + this.state.settings.iconSpacing + 'px;';
+        this.actor.style = existingStyle + 'margin-' + direction + ':6px;';
     }
 
     updateIconBoxClip() {
@@ -279,7 +282,7 @@ class AppGroup {
         let icon;
 
         if (this.groupState.app) {
-            if (metaWindow) {
+            if (metaWindow && (!this.state.settings.groupApps || this.groupState.app.is_window_backed())) {
                 icon = this.groupState.app.create_icon_texture_for_window(this.iconSize, metaWindow);
             } else {
                 icon = this.groupState.app.create_icon_texture(this.iconSize);
@@ -325,7 +328,7 @@ class AppGroup {
 
         this.actor.remove_style_pseudo_class('active');
         this.actor.add_style_class_name('grouped-window-list-item-demands-attention');
-        if (counter < 4) {
+        if (counter < FLASH_MAX_COUNT) {
             setTimeout(() => {
                 if (this.actor && this.actor.has_style_class_name('grouped-window-list-item-demands-attention')) {
                     this.actor.remove_style_class_name('grouped-window-list-item-demands-attention');
@@ -341,13 +344,12 @@ class AppGroup {
     getPreferredWidth(actor, forHeight, alloc) {
         let [iconMinSize, iconNaturalSize] = this.iconBox.get_preferred_width(forHeight);
         let [labelMinSize, labelNaturalSize] = this.label.get_preferred_width(forHeight);
-        let {iconSpacing} = this.state.settings;
         // The label text starts in the center of the icon, so we should allocate the space
         // needed for the icon plus the space needed for(label - icon/2)
-        alloc.min_size = iconNaturalSize + iconSpacing;
+        alloc.min_size = iconNaturalSize + 6;
         if (this.state.orientation === St.Side.TOP || this.state.orientation === St.Side.BOTTOM) {
             let max = this.labelVisible && this.groupState.metaWindows.length > 0 ?
-                labelNaturalSize + iconNaturalSize + iconSpacing : 0;
+                labelNaturalSize + iconNaturalSize + 6 : 0;
             alloc.natural_size = Math.min(iconNaturalSize + Math.max(max, labelNaturalSize), MAX_BUTTON_WIDTH * global.ui_scale);
         } else {
             alloc.natural_size = this.state.trigger('getPanelHeight');
@@ -366,7 +368,6 @@ class AppGroup {
         let allocHeight = box.y2 - box.y1;
         let childBox = new Clutter.ActorBox();
         let direction = this.actor.get_text_direction();
-        let {iconSpacing} = this.state.settings;
         let spacing = 0;
 
         // Set the icon to be left-justified (or right-justified) and centered vertically
@@ -378,7 +379,7 @@ class AppGroup {
 
         if (this.labelVisible && this.groupState.metaWindows.length > 0) {
             if (direction === Clutter.TextDirection.LTR) {
-                childBox.x1 = box.x1 + iconSpacing;
+                childBox.x1 = box.x1 + 6;
             } else {
                 childBox.x1 = Math.max(box.x1, box.x2 - naturalWidth);
             }
@@ -436,13 +437,7 @@ class AppGroup {
             });
         }
 
-        if (this.progressOverlay.visible) {
-            childBox.x1 = 0;
-            childBox.y1 = 0;
-            childBox.x2 = Math.max((this.actor.width) * (this._progress / 100.0), 1.0);
-            childBox.y2 = this.actor.height;
-            this.progressOverlay.allocate(childBox, flags);
-        }
+        if (this.progressOverlay.visible) this.allocateProgress(childBox, flags);
     }
 
     showLabel(animate = false) {
@@ -502,6 +497,8 @@ class AppGroup {
     }
 
     onLeave() {
+        this.groupState.pressed = false;
+
         if (this.state.panelEditMode) return false;
 
         if (this.hoverMenu) this.hoverMenu.onMenuLeave();
@@ -538,15 +535,38 @@ class AppGroup {
         }
     }
 
+    averageProgress() {
+        let {metaWindows} = this.groupState;
+        let total = 0;
+        let count = 0;
+        each(metaWindows, function(metaWindow) {
+            let {progress} = metaWindow;
+            if (progress < 1) return;
+            total += progress;
+            count++;
+        });
+        return total / count;
+    }
+
+    allocateProgress(childBox = null, flags = 0) {
+        if (!childBox) childBox = new Clutter.ActorBox();
+        childBox.x1 = 0;
+        childBox.y1 = 0;
+        childBox.x2 = Math.max((this.actor.width) * (this.progress / 100.0), 1.0);
+        childBox.y2 = this.actor.height;
+        this.progressOverlay.allocate(childBox, flags);
+    }
+
     onProgressChange(metaWindow) {
-        if (metaWindow.progress !== this._progress) {
-            this._progress = metaWindow.progress;
-            if (this._progress > 0) {
-                this.progressOverlay.show();
+        let progress = this.averageProgress();
+        if (progress !== this.progress) {
+            this.progress = progress;
+            if (this.progress > 0) {
+                if (!this.progressOverlay.visible) this.progressOverlay.show();
+                this.allocateProgress();
             } else {
                 this.progressOverlay.hide();
             }
-            this.actor.queue_relayout();
         }
     }
 
@@ -605,25 +625,18 @@ class AppGroup {
             this._draggable._overrideX = Math.round(x);
             this._draggable._overrideY = null;
         }
+
+        if (this.rightClickMenu) this.rightClickMenu.close(false);
         if (this.hoverMenu) this.groupState.trigger('hoverMenuClose');
     }
 
-    onDragEnd() {
-        if (this.rightClickMenu) this.rightClickMenu.close(false);
-        if (this.hoverMenu) this.hoverMenu.close(false);
-        this.listState.trigger('updateAppGroupIndexes', this.groupState.appId);
-        this.state.trigger('clearDragPlaceholder');
-    }
-
     onDragCancelled() {
-        if (this.rightClickMenu) this.rightClickMenu.close(false);
-        if (this.hoverMenu) this.hoverMenu.close(false);
-        this.state.trigger('clearDragPlaceholder');
+        this.state.trigger('moveLauncher', this);
     }
 
     handleDragOver(source, actor, x, y, time) {
         if (!this.state.settings.enableDragging
-            || source instanceof AppGroup
+            || actor.name != "xdnd-proxy-actor"
             || this.state.panelEditMode) {
             return DND.DragMotionResult.CONTINUE;
         }
@@ -641,7 +654,7 @@ class AppGroup {
                 this.hoverMenu.open(true);
             }
         }
-        return true;
+        return DND.DragMotionResult.CONTINUE;
     }
 
     getDragActor() {
@@ -659,13 +672,24 @@ class AppGroup {
         this.badge.show();
     }
 
-    launchNewInstance() {
-        this.groupState.app.open_new_window(-1);
+    launchNewInstance(offload=false) {
+        if (offload) {
+            try {
+                this.groupState.app.launch_offloaded(0, [], -1);
+            } catch (e) {
+                logError(e, "Could not launch app with dedicated gpu: ");
+            }
+        } else {
+            this.groupState.app.open_new_window(-1);
+        }
+
         this.animate();
     }
 
     onAppButtonRelease(actor, event) {
-        this.state.trigger('clearDragPlaceholder');
+        if (!this.groupState.pressed) {
+            return;
+        }
 
         let button = event.get_button();
         let nWindows = this.groupState.metaWindows.length;
@@ -726,7 +750,24 @@ class AppGroup {
                 return;
             }
             if (this.state.settings.leftClickAction === 3 && nWindows > 1) {
-                this.state.trigger('cycleWindows', null, this.actor._delegate);
+                let foundActive = false;
+                for (let i = 0, len = nWindows; i < len; i++) {
+                    if (
+                        this.groupState.lastFocused &&
+                        this.groupState.metaWindows[i] === this.groupState.lastFocused
+                    ) {
+                        if (this.groupState.metaWindows[i].appears_focused) {
+                            this.state.trigger("cycleWindows", null, this.actor._delegate);
+                        } else {
+                            handleMinimizeToggle(this.groupState.metaWindows[i]);
+                        }
+                        foundActive = true;
+                        break;
+                    }
+                }
+                if (!foundActive) {
+                    handleMinimizeToggle(this.groupState.metaWindows[0]);
+                }
                 return;
             }
             if (this.hoverMenu) this.hoverMenu.shouldOpen = false;
@@ -765,8 +806,10 @@ class AppGroup {
 
     onAppButtonPress(actor, event) {
         let button = event.get_button();
+        this.groupState.pressed = true;
 
         if (button === 3) return true;
+
         return false;
     }
 
@@ -791,7 +834,9 @@ class AppGroup {
                     this.appKeyTimeout = 0;
                     return;
                 }
-                this.hoverMenu.close(true);
+                if (this.hoverMenu) {
+                    this.hoverMenu.close(true);
+                }
                 this.appKeyTimeout = 0;
             }, this.state.settings.showAppsOrderTimeout);
         }
@@ -837,19 +882,22 @@ class AppGroup {
             this.signals.connect(metaWindow, 'notify::gtk-application-id', (w) => this.onAppChange(w));
             this.signals.connect(metaWindow, 'notify::wm-class', (w) => this.onAppChange(w));
 
-            if (!this.state.settings.groupApps) {
-                this.signals.connect(metaWindow, 'notify::icon', (w) => this.setIcon(w));
-            }
+            this.signals.connect(metaWindow, 'icon-changed', (w) => this.setIcon(w));
 
             if (metaWindow.progress !== undefined) {
-                this._progress = metaWindow.progress;
+                // Check if GWL is starting with pre-existing windows that have progress,
+                // and defer to the next tick in case the actor isn't on the stage yet.
+                if (metaWindow.progress > 0 || this.progress > 0) {
+                    setTimeout(() => this.onProgressChange(), 0);
+                } else {
+                    this.progress = 0;
+                }
                 this.signals.connect(metaWindow, 'notify::progress', () => this.onProgressChange(metaWindow));
             }
 
             // Set the initial button label as not all windows will get updated via signals initially.
             if (this.state.settings.titleDisplay > 1) {
                 this.onWindowTitleChanged(metaWindow);
-                this.state.trigger('updateThumbnailsStyle');
             }
             if (refWindow === -1) {
                 metaWindows.push(metaWindow);
@@ -875,11 +923,9 @@ class AppGroup {
 
         this.groupState.metaWindows.splice(refWindow, 1);
 
+        if (this.progressOverlay.visible) this.onProgressChange();
+
         if (this.groupState.metaWindows.length > 0 && !this.groupState.willUnmount) {
-            if (this.progressOverlay.visible && metaWindow.progress > 0) {
-                this._progress = 0;
-                this.progressOverlay.visible = false;
-            }
             this.onWindowTitleChanged(this.groupState.lastFocused);
             this.groupState.set({
                 metaWindows: this.groupState.metaWindows,
@@ -992,8 +1038,6 @@ class AppGroup {
                 && this.groupState.appId.indexOf(global.display.focus_window.wm_class.toLowerCase()) === -1) {
                 this.hideLabel();
             }
-            // Re-orient the menu after the focus button expands
-            if (this.hoverMenu) this.hoverMenu.setStyleOptions(false);
         }
     }
 
@@ -1041,7 +1085,7 @@ class AppGroup {
         });
     }
 
-    animate(step = 0) {
+    animate() {
         let effect = this.state.settings.launcherAnimationEffect;
 
         if (effect === 1) return;
@@ -1049,24 +1093,22 @@ class AppGroup {
             this.iconBox.set_z_rotation_from_gravity(0.0, Clutter.Gravity.CENTER);
             Tweener.addTween(this.iconBox, {
                 opacity: 70,
-                time: 1.0,
+                time: 0.2,
                 transition: 'linear',
                 onCompleteScope: this,
                 onComplete() {
                     Tweener.addTween(this.iconBox, {
                         opacity: 255,
-                        time: 0.5,
+                        time: 0.2,
                         transition: 'linear'
                     });
                 }
             });
         } else if (effect === 3) {
-            if (step >= 3) return;
-
             this.iconBox.set_pivot_point(0.5, 0.5);
             Tweener.addTween(this.iconBox, {
-                scale_x: 0.7,
-                scale_y: 0.7,
+                scale_x: 0.8,
+                scale_y: 0.8,
                 time: 0.2,
                 transition: 'easeOutQuad',
                 onComplete: () => {
@@ -1074,10 +1116,7 @@ class AppGroup {
                         scale_x: 1.0,
                         scale_y: 1.0,
                         time: 0.2,
-                        transition: 'easeOutQuad',
-                        onComplete: () => {
-                            this.animate(step + 1);
-                        }
+                        transition: 'easeOutQuad'
                     });
                 }
             });
