@@ -10,11 +10,10 @@ try:
     import html
     import subprocess
     import threading
-    import time
-    import dbus
     from PIL import Image
     import datetime
     import proxygsettings
+    import time
 except Exception as detail:
     print(detail)
     sys.exit(1)
@@ -162,7 +161,6 @@ class Spice_Harvester(GObject.Object):
         else:
             self.settings = Gio.Settings.new('org.cinnamon')
             self.enabled_key = 'enabled-%ss' % self.collection_type
-        self.settings.connect('changed::%s' % self.enabled_key, self._update_status)
 
         if self.themes:
             self.install_folder = '%s/.themes/' % (home)
@@ -170,6 +168,9 @@ class Spice_Harvester(GObject.Object):
         else:
             self.install_folder = '%s/.local/share/cinnamon/%ss/' % (home, self.collection_type)
             self.spices_directories = ('/usr/share/cinnamon/%ss/' % self.collection_type, self.install_folder)
+            self.settings.connect('changed::%s' % self.enabled_key, self._update_status)
+
+        self._update_status()
 
         self._load_metadata()
 
@@ -190,8 +191,8 @@ class Spice_Harvester(GObject.Object):
         try:
             Gio.DBusProxy.new_for_bus(Gio.BusType.SESSION, Gio.DBusProxyFlags.NONE, None,
                                       'org.Cinnamon', '/org/Cinnamon', 'org.Cinnamon', None, self._on_proxy_ready, None)
-        except dbus.exceptions.DBusException as e:
-            print(e)
+        except GLib.Error as e:
+            print(e.message)
 
     def _on_proxy_ready (self, object, result, data=None):
         try:
@@ -214,6 +215,9 @@ class Spice_Harvester(GObject.Object):
                 self.send_deferred_proxy_calls()
                 return
 
+        if signal_name == "XletsLoadedComplete":
+            self._update_status()
+
         for name, callback in self._proxy_signals:
             if signal_name == name:
                 callback(*params)
@@ -357,6 +361,9 @@ class Spice_Harvester(GObject.Object):
             self._directory_changed()
 
     def _download(self, out_file, url, binary=True):
+        timestamp = round(time.time())
+        url = "%s?time=%d" % (url, timestamp)
+        print("Downloading from %s" % url)
         try:
             open_args = 'wb' if binary else 'w'
             with open(out_file, open_args) as outfd:
@@ -389,7 +396,8 @@ class Spice_Harvester(GObject.Object):
             else:
                 connection = HTTPSConnection(host, timeout=15)
             headers = { "Accept-Encoding": "identity", "Host": host, "User-Agent": "Python/3" }
-            connection.request("GET", parsed_url.path, headers=headers)
+            full_path = "%s?%s" % (parsed_url.path, parsed_url.query)
+            connection.request("GET", full_path, headers=headers)
             urlobj = connection.getresponse()
             assert urlobj.getcode() == 200
 
@@ -458,6 +466,7 @@ class Spice_Harvester(GObject.Object):
         if not self.themes:
             enabled_list = self.settings.get_strv(self.enabled_key)
             for item in enabled_list:
+                item = item.replace("!", "")
                 if uuid in item.split(":"):
                     enabled_count += 1
         elif self.settings.get_string(self.enabled_key) == uuid:
@@ -575,13 +584,9 @@ class Spice_Harvester(GObject.Object):
         self.download_total_files = 0
         self.download_current_file = 0
         self.is_downloading_image_cache = False
-        self.settings.set_int('%s-cache-updated' % self.collection_type, time.time())
         self._advance_queue()
         self.emit('cache-loaded')
 
-    def get_cache_age(self):
-        return (time.time() - self.settings.get_int('%s-cache-updated' % self.collection_type)) / 86400
-
     # checks for corrupt images in the cache so we can redownload them the next time we refresh
     def _is_bad_image(self, path):
         try:
